/*
Autor: Marius M.
Quelle: www.montisblog.de
Funktion: Dieses Programm liest einen Text aus, der im Seriellen Monitor der Arduino
IDE eingegeben wurde und gibt ihn Buchstabe für Buchstabe wieder aus. Dabei wird nicht
auf groß- und kleinschreibung geachtet und es können nur Buchstaben von a bis z und
von A bis Z eingegeben werden.

Layout der LED Matrix:

           
   Spalte       1  2  3  4  5
       D Pin    13 10 7  5  2
               ---------------
         1 11 | 0  1  2  3  4 
         2 9  | 5  6  7  8  9 
 Zeile   3 8  | 10 11 12 13 14
         4 6  | 15 16 17 18 19
         5 3  | 20 21 22 23 24
 
*/


//In diesen beiden Arrays sind die Arduino Pins gespeichert, die eine Spalte bzw. 
//Zeile ansteuern.
const int Zeile[5]={
  11,9,8,6,3};
const int Spalte[5]={
  13,10,7,5,2};

//Mit den beiden folgenden Arrays kann festgestellt werden, welche Pins man ansteuern muss, um
//eine bestimmte LED zum leuchten zu bringen.
//Beispiel: LED 8 soll leuchten:
//Es müssen angesteuert werden Pin D5 (Spalte) und Pin D9(Zeile)
const int PosSpalte[25]={
  13,10,7,5,2,13,10,7,5,2,13,10,7,5,2,13,10,7,5,2,13,10,7,5,2};
const int PosZeile[25]={
  11,11,11,11,11,9,9,9,9,9,8,8,8,8,8,6,6,6,6,6,3,3,3,3,3};

//Diese zweidimensionale Array beschreibt, welche LEDs leuchten müssen, damit ein 
//bestimmter Buchstabe erscheint
const int Alpha [26] [14] = {
  {
    1,2,3,6,8,11,12,13,16,18,21,23          }
  ,        //A
  {
    1,2,3,6,8,11,12,16,18,21,22,23          }
  ,        //B
  {
    1,2,3,6,11,16,21,22,23          }
  ,                //C
  {
    0,1,2,5,8,10,13,15,18,20,21,22          }
  ,        //D
  {
    0,5,10,15,20,1,2,3,11,12,13,21,22,23          }
  ,  //E
  {
    0,5,10,15,20,1,2,3,11,12,13          }
  ,           //F
  {
    0,5,10,15,20,1,2,3,12,13,18,21,22,23          }
  ,  //G
  {
    0,5,10,15,20,11,12,13,3,8,18,23          }
  ,       //H
  {
    1,2,3,7,12,17,22,21,23          }
  ,                //I
  {
    0,1,2,3,8,13,15,18,21,22          }
  ,              //J
  {
    0,5,10,15,20,3,7,11,17,23          }
  ,             //K
  {
    0,5,10,15,20,21,22,23          }
  ,                 //L
  {
    0,4,5,6,8,9,10,12,14,15,19,20,24          }
  ,      //M
  {
    0,5,10,15,20,6,12,18,24,19,14,9,4          }
  ,     //N
  {
    1,2,5,10,15,8,13,18,21,22          }
  ,             //O
  {
    0,5,10,15,20,1,2,8,11,12          }
  ,              //P
  {
    1,2,5,10,15,8,13,18,21,22,23,24          }
  ,       //Q
  {
    0,5,10,15,20,1,2,8,11,12,17,23          }
  ,        //R
  {
    3,2,1,5,11,12,18,22,21,20          }
  ,             //S
  {
    0,1,2,3,4,7,12,17,22          }
  ,                  //T
  {
    0,5,10,15,21,22,18,13,8,3          }
  ,             //U
  {
    0,5,11,16,22,18,13,9,4          }
  ,                //V
  {
    0,5,10,16,21,2,7,12,18,23,4,9,14          }
  ,      //W
  {
    0,6,12,18,24,4,8,16,20          }
  ,                //X
  {
    0,6,12,4,8,17,22          }
  ,                      //Y
  {
    0,1,2,3,4,8,12,16,20,21,22,23,24          }
  ,      //Z
}; 



int Pos;
int z;
int laenge;
int Eing;

long start = 0;
long akt = 0;
long zeit = 0;
long dauer = 1000; //Die Zeit, die ein Buchstabe angezeigt wird.

void setup()
{
  Serial.begin(9600);

  //Alle benötigten Pins werden als Output deklariert.
  for (int i = 0; i < 5; i++) {
    pinMode(Zeile[i], OUTPUT);
    pinMode(Spalte[i], OUTPUT);
  }


}

void loop()
{
  //Von hier...
  if (Serial.available() > 0) {
    Eing = Serial.read();
    Serial.println(Eing);
    if (Eing < 123 && Eing > 96){ 
      Eing = Eing - 97;
    } 
    else if (Eing < 91 && Eing > 64){
      Eing = Eing - 65;
    }
   //Bis hier einlesen der Zeichen aus dem Seriellen Monitor und
   //Umrechnen in eine Nummer von 0 bis 25 (Buchstaben des Alphabets


   //Von hier...
    start = millis(); //Merken der Anfangszeit
    laenge = sizeof(Alpha[Eing]) / 2;

    //Solange der aktuelle Buchstabe weniger als "dauer" angezeigt wurde, tue folgendes:
    while (akt < dauer){ 

      zeit = millis();
      akt = zeit - start;

      for (int j = 0; j < laenge; j++)
      {
        zeit = millis();
        Pos = Alpha[Eing][j];

        if ((j > 5 && Pos != 0) || j < 6){
          digitalWrite(PosSpalte[Pos], 0);
          for (int i = 0; i < 5; i++) {
            if (PosSpalte[Pos] != Spalte[i]) {
              digitalWrite(Spalte[i], 1);
            }
          }

          digitalWrite(PosZeile[Pos], 1);
          for (int i = 0; i < 5; i++) {
            if (PosZeile[Pos] != Zeile[i]) {
              digitalWrite(Zeile[i], 0);
            }
          }

        }

        delay(1);

      }

    }
    //Bis hier ausgeben der Buchstaben
  } 
  
  //Wenn keine Eingabe am Computer erfolgt, sollen alle LEDs aus sein.
  else {
    for (int i = 0; i < 5; i++) {
      digitalWrite(Zeile[i], LOW);
      digitalWrite(Spalte[i], LOW);
    }

  }


  start = 0;
  akt = 0;
  zeit = 0;

}



